@@ -62,7 +62,6 @@
 #include <net/if.h>
 
 #include <nuttx/can.h>
-#include <netpacket/can.h>
 
 #include "terminal.h"
 #include "lib.h"
@@ -99,7 +98,6 @@
 const char col_on [MAXCOL][19] = {BLUE, RED, GREEN, BOLD, MAGENTA, CYAN};
 const char col_off [] = ATTRESET;
 
-static char *cmdlinename[MAXSOCK];
 static __u32 dropcnt[MAXSOCK];
 static __u32 last_dropcnt[MAXSOCK];
 static char devname[MAXIFNAMES][IFNAMSIZ+1];
@@ -229,7 +227,7 @@ int main(int argc, char **argv)
 	unsigned char logfrmt = 0;
 	int count = 0;
 	int rcvbuf_size = 0;
-	int opt, ret;
+	int opt;
 	int currmax, numfilter;
 	int join_filter;
 	char *ptr, *nptr;
@@ -394,8 +392,6 @@ int main(int argc, char **argv)
 			return 1;
 		}
 
-		cmdlinename[i] = ptr; /* save pointer to cmdline name of this socket */
-
 		if (nptr)
 			nbytes = nptr - ptr;  /* interface name is up the first ',' */
 		else
@@ -618,7 +614,7 @@ int main(int argc, char **argv)
 		if (timeout_current)
 			*timeout_current = timeout_config;
 
-		if ((ret = select(s[currmax-1]+1, &rdfs, NULL, NULL, timeout_current)) <= 0) {
+		if ((select(s[currmax-1]+1, &rdfs, NULL, NULL, timeout_current)) <= 0) {
 			//perror("select");
 			running = 0;
 			continue;
