@@ -43,7 +43,7 @@
 #  define CONFIG_EXAMPLES_CALIB_UDELAY_NUM_RESULTS 20
 #endif
 
-#define DELAY_TEST_ITERS 100000
+#define DELAY_TEST_ITERS CONFIG_EXAMPLES_CALIB_UDELAY_DELAY_TEST_ITERATIONS
 
 /****************************************************************************
  * Private Types
