@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <getopt.h>
+#include <inttypes.h>
 
 #include "foc_debug.h"
 #include "foc_thr.h"
@@ -45,6 +46,21 @@
 #define OPT_IIV     (SCHAR_MAX + 6)
 #define OPT_IIS     (SCHAR_MAX + 7)
 
+#define OPT_VOPLLKP (SCHAR_MAX + 8)
+#define OPT_VOPLLKI (SCHAR_MAX + 9)
+#define OPT_VODIVS  (SCHAR_MAX + 10)
+#define OPT_VODIVF  (SCHAR_MAX + 11)
+
+#define OPT_VCPIKP  (SCHAR_MAX + 12)
+#define OPT_VCPIKI  (SCHAR_MAX + 13)
+
+#define OPT_ANFOS   (SCHAR_MAX + 14)
+#define OPT_ANFOG   (SCHAR_MAX + 15)
+
+#define OPT_OLFORCE (SCHAR_MAX + 16)
+#define OPT_OLTHR   (SCHAR_MAX + 17)
+#define OPT_OLHYS   (SCHAR_MAX + 18)
+
 /****************************************************************************
  * Private Data
  ****************************************************************************/
@@ -60,6 +76,8 @@ static struct option g_long_options[] =
 #endif
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_VEL
     { "vel", required_argument, 0, 'v' },
+    { "acc", required_argument, 0, 'a' },
+    { "dec", required_argument, 0, 'd' },
 #endif
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_POS
     { "pos", required_argument, 0, 'x' },
@@ -68,6 +86,11 @@ static struct option g_long_options[] =
     { "en", required_argument, 0, 'j' },
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_OPENLOOP
     { "oqset", required_argument, 0, 'o' },
+    { "olforce", no_argument, 0, OPT_OLFORCE },
+#  ifdef CONFIG_EXAMPLES_FOC_ANGOBS
+    { "olthr", required_argument, 0, OPT_OLTHR },
+    { "olhys", required_argument, 0, OPT_OLHYS },
+#  endif
 #endif
 #ifdef CONFIG_EXAMPLES_FOC_CONTROL_PI
     { "fkp", required_argument, 0, OPT_FKP },
@@ -79,6 +102,22 @@ static struct option g_long_options[] =
     { "irs", required_argument, 0, OPT_IRS },
     { "iiv", required_argument, 0, OPT_IIV },
     { "iis", required_argument, 0, OPT_IIS },
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_VELOBS_PLL
+    { "vopllkp", required_argument, 0, OPT_VOPLLKP },
+    { "vopllki", required_argument, 0, OPT_VOPLLKI },
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_VELOBS_DIV
+    { "vodivs", required_argument, 0, OPT_VODIVS },
+    { "vodivf", required_argument, 0, OPT_VODIVF },
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_VELCTRL_PI
+    { "vcpikp", required_argument, 0, OPT_VCPIKP },
+    { "vcpiki", required_argument, 0, OPT_VCPIKI },
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_ANGOBS_NFO
+    { "anfos", required_argument, 0, OPT_ANFOS },
+    { "anfog", required_argument, 0, OPT_ANFOG },
 #endif
     { 0, 0, 0, 0 }
   };
@@ -91,7 +130,7 @@ static struct option g_long_options[] =
  * Name: foc_help
  ****************************************************************************/
 
-static void foc_help(void)
+static void foc_help(FAR struct args_s *args)
 {
   PRINTF("Usage: foc [OPTIONS]\n");
   PRINTF("  [-t] run time (default: %d)\n",
@@ -120,38 +159,73 @@ static void foc_help(void)
 #endif
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_VEL
   PRINTF("  [-v] velocity [x1000]\n");
+  PRINTF("  [-a] acceleration [x1000] (default: %" PRId32 ")\n",
+         args->cfg.acc);
+  PRINTF("  [-d] deceleration [x1000] (default: %" PRId32 ")\n",
+         args->cfg.dec);
 #endif
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_POS
   PRINTF("  [-x] position [x1000]\n");
 #endif
   PRINTF("  [-s] motor state init (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_STATE_INIT);
+         args->state);
   PRINTF("       1 - motor free\n");
   PRINTF("       2 - motor stop\n");
   PRINTF("       3 - motor CW\n");
   PRINTF("       4 - motor CCW\n");
   PRINTF("  [-j] enable specific instances\n");
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_OPENLOOP
-  PRINTF("  [-o] openloop Vq/Iq setting [x1000] (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_OPENLOOP_Q);
+  PRINTF("  [-o] openloop Vq/Iq setting [x1000] (default: %" PRId32 ")\n",
+         args->cfg.qparam);
+  PRINTF("  [--olforce] force openloop\n");
+#  ifdef CONFIG_EXAMPLES_FOC_ANGOBS
+  PRINTF("  [--olthr] observer vel threshold [x1] (default: %d)\n",
+         CONFIG_EXAMPLES_FOC_ANGOBS_THR);
+  PRINTF("  [--olhys] observer vel hysteresys [x1] (default: %d)\n",
+         CONFIG_EXAMPLES_FOC_ANGOBS_THR);
+#  endif
 #endif
 #ifdef CONFIG_EXAMPLES_FOC_CONTROL_PI
-  PRINTF("  [--fki] PI Kp coefficient [x1000] (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_IDQ_KP);
-  PRINTF("  [--fkp] PI Ki coefficient [x1000] (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_IDQ_KI);
+  PRINTF("  [--fkp] PI Kp coefficient [x1000] (default: %" PRId32 ")\n",
+         args->cfg.foc_pi_kp);
+  PRINTF("  [--fki] PI Ki coefficient [x1000] (default: %" PRId32 ")\n",
+         args->cfg.foc_pi_ki);
 #endif
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_IDENT
-  PRINTF("  [--irki] res Ki coefficient [x1000] (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_IDENT_RES_KI);
-  PRINTF("  [--irc] res current [x1000] (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_IDENT_RES_CURRENT);
-  PRINTF("  [--irs] res sec (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_IDENT_RES_SEC);
-  PRINTF("  [--iiv] ind voltage [x1000] (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_IDENT_IND_VOLTAGE);
-  PRINTF("  [--iis] ind sec (default: %d)\n",
-         CONFIG_EXAMPLES_FOC_IDENT_IND_SEC);
+  PRINTF("  [--irki] res Ki coefficient [x1000] (default: %" PRId32 ")\n",
+         args->cfg.ident_res_ki);
+  PRINTF("  [--irc] res current [x1000] (default: %" PRId32 ")\n",
+         args->cfg.ident_res_curr);
+  PRINTF("  [--irs] res sec (default: %" PRId32 ")\n",
+         args->cfg.ident_res_sec);
+  PRINTF("  [--iiv] ind voltage [x1000] (default: %" PRId32 ")\n",
+         args->cfg.ident_ind_volt);
+  PRINTF("  [--iis] ind sec (default: %" PRId32 ")\n",
+         args->cfg.ident_ind_sec);
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_VELOBS_PLL
+  PRINTF("  [--vopllkp] velobs PLL Kp [x1] (default: %" PRId32 ")\n",
+         args->cfg.vel_pll_kp);
+  PRINTF("  [--vopllki] velobs PLL Ki [x1] (default: %" PRId32 ")\n",
+         args->cfg.vel_pll_ki);
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_VELOBS_DIV
+  PRINTF("  [--vodivs] velobs DIV samples (default: %" PRId32 ")\n",
+         args->cfg.vel_div_samples);
+  PRINTF("  [--vodivf] velobs DIV filter (default: %" PRId32 ")\n",
+         args->cfg.vel_div_filter);
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_VELCTRL_PI
+  PRINTF("  [--vcpikp] velctrl PI Kp [x1000000] (default: %" PRId32 ")\n",
+         args->cfg.vel_pi_kp);
+  PRINTF("  [--vcpiki] velctrl PI Ki [x1000000] (default: %" PRId32 ")\n",
+         args->cfg.vel_pi_ki);
+#endif
+#ifdef CONFIG_EXAMPLES_FOC_ANGOBS_NFO
+  PRINTF("  [--anfos] angobs NFO Slow [x1] (default: %" PRId32 ")\n",
+         args->cfg.ang_nfo_slow);
+  PRINTF("  [--anfog] angobs NFO Gain [x1] (default: %" PRId32 ")\n",
+         args->cfg.ang_nfo_gain);
 #endif
 }
 
@@ -170,7 +244,7 @@ void parse_args(FAR struct args_s *args, int argc, FAR char **argv)
 
   while (1)
     {
-      c = getopt_long(argc, argv, "ht:f:m:o:r:v:x:s:j:", g_long_options,
+      c = getopt_long(argc, argv, "ht:f:m:o:r:v:a:d:x:s:j:", g_long_options,
                       &option_index);
 
       if (c == -1)
@@ -226,6 +300,62 @@ void parse_args(FAR struct args_s *args, int argc, FAR char **argv)
             }
 #endif
 
+#ifdef CONFIG_EXAMPLES_FOC_VELOBS_PLL
+          case OPT_VOPLLKP:
+            {
+              args->cfg.vel_pll_kp = atoi(optarg);
+              break;
+            }
+
+          case OPT_VOPLLKI:
+            {
+              args->cfg.vel_pll_ki = atoi(optarg);
+              break;
+            }
+#endif
+
+#ifdef CONFIG_EXAMPLES_FOC_VELOBS_DIV
+          case OPT_VODIVS:
+            {
+              args->cfg.vel_div_samples = atoi(optarg);
+              break;
+            }
+
+          case OPT_VODIVF:
+            {
+              args->cfg.vel_div_filter = atoi(optarg);
+              break;
+            }
+#endif
+
+#ifdef CONFIG_EXAMPLES_FOC_VELCTRL_PI
+          case OPT_VCPIKP:
+            {
+              args->cfg.vel_pi_kp = atoi(optarg);
+              break;
+            }
+
+          case OPT_VCPIKI:
+            {
+              args->cfg.vel_pi_ki = atoi(optarg);
+              break;
+            }
+#endif
+
+#ifdef CONFIG_EXAMPLES_FOC_ANGOBS_NFO
+          case OPT_ANFOS:
+            {
+              args->cfg.ang_nfo_slow = atoi(optarg);
+              break;
+            }
+
+          case OPT_ANFOG:
+            {
+              args->cfg.ang_nfo_gain = atoi(optarg);
+              break;
+            }
+#endif
+
           case 't':
             {
               args->time = atoi(optarg);
@@ -234,7 +364,7 @@ void parse_args(FAR struct args_s *args, int argc, FAR char **argv)
 
           case 'h':
             {
-              foc_help();
+              foc_help(args);
               exit(0);
             }
 
@@ -264,6 +394,18 @@ void parse_args(FAR struct args_s *args, int argc, FAR char **argv)
               args->cfg.velmax = atoi(optarg);
               break;
             }
+
+          case 'a':
+            {
+              args->cfg.acc = atoi(optarg);
+              break;
+            }
+
+          case 'd':
+            {
+              args->cfg.dec = atoi(optarg);
+              break;
+            }
 #endif
 
 #ifdef CONFIG_EXAMPLES_FOC_HAVE_POS
@@ -292,6 +434,26 @@ void parse_args(FAR struct args_s *args, int argc, FAR char **argv)
               args->cfg.qparam = atoi(optarg);
               break;
             }
+
+          case OPT_OLFORCE:
+            {
+              args->cfg.ol_force = true;
+              break;
+            }
+
+#  ifdef CONFIG_EXAMPLES_FOC_ANGOBS
+          case OPT_OLTHR:
+            {
+              args->cfg.ol_thr = atoi(optarg);
+              break;
+            }
+
+          case OPT_OLHYS:
+            {
+              args->cfg.ol_hys = atoi(optarg);
+              break;
+            }
+#  endif
 #endif
 
           case '?':
@@ -307,7 +469,7 @@ void parse_args(FAR struct args_s *args, int argc, FAR char **argv)
                   PRINTF("ERROR: invalid option %s\n", argv[optind - 1]);
                 }
 
-              foc_help();
+              foc_help(args);
               exit(1);
             }
         }
