@@ -523,7 +523,8 @@ int main(int argc, FAR char *argv[])
   for (i = 0; i < CONFIG_PWM_NCHANNELS; i++)
     {
       info.channels[i].channel = g_pwmstate.channels[i];
-      info.channels[i].duty = b16divi(uitoub16(g_pwmstate.duties[i]), 100);
+      info.channels[i].duty = g_pwmstate.duties[i] ? \
+        b16divi(uitoub16(g_pwmstate.duties[i]) - 1, 100) : 0;
       printf(" channel: %d duty: %08" PRIx32,
         info.channels[i].channel, info.channels[i].duty);
     }
@@ -531,7 +532,8 @@ int main(int argc, FAR char *argv[])
   printf("\n");
 
 #else
-  info.duty  = b16divi(uitoub16(g_pwmstate.duty), 100);
+  info.duty = g_pwmstate.duty ? \
+    b16divi(uitoub16(g_pwmstate.duty) - 1, 100) : 0;
 #  ifdef CONFIG_PWM_PULSECOUNT
   info.count = g_pwmstate.count;
 
