@@ -1,5 +1,5 @@
 /****************************************************************************
- * apps/usrsocktest/usrsocktest_remote_disconnect.c
+ * apps/examples/usrsocktest/usrsocktest_remote_disconnect.c
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -977,13 +977,13 @@ TEST_TEAR_DOWN(remote_disconnect)
   if (sd >= 0)
     {
       ret = close(sd);
-      assert(ret >= 0);
+      TEST_ASSERT_TRUE(ret >= 0);
     }
 
   if (started)
     {
       ret = usrsocktest_daemon_stop();
-      assert(ret == OK);
+      TEST_ASSERT_EQUAL(ret, OK);
     }
 }
 
