@@ -1,5 +1,5 @@
 /****************************************************************************
- * apps/graphics/nuttx/pdcscrn.c
+ * apps/graphics/pdcurs34/nuttx/pdcscrn.c
  *
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
@@ -31,6 +31,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <graphics/curses.h>
 #include "pdcnuttx.h"
@@ -61,7 +62,8 @@ bool graphic_screen = false;
 #ifdef CONFIG_SYSTEM_TERMCURSES
 static void PDC_scr_free_term(FAR SCREEN *sp)
 {
-  FAR struct pdc_termscreen_s *termscreen = (FAR struct pdc_termscreen_s *)sp;
+  FAR struct pdc_termscreen_s *termscreen =
+    (FAR struct pdc_termscreen_s *)sp;
   FAR struct pdc_termstate_s  *termstate = &termscreen->termstate;
 
   /* Deinitialize termcurses */
@@ -145,7 +147,8 @@ static int PDC_scr_open_term(int argc, char **argv)
   ret = termcurses_getwinsize(termstate->tcurs, &winsz);
   if (ret != OK)
     {
-      PDC_LOG(("ERROR: Terminal termcurses driver doesn't support size reporting\n"));
+      PDC_LOG(("ERROR: Terminal termcurses driver doesn't support size "
+               "reporting\n"));
       free(termscreen);
       return ERR;
     }
@@ -156,8 +159,8 @@ static int PDC_scr_open_term(int argc, char **argv)
   SP->cols                     = winsz.ws_col;
   termscreen->termstate.out_fd = 1;
   termscreen->termstate.in_fd  = 0;
-  termscreen->termstate.fg_red = 0xFFFE;
-  termscreen->termstate.bg_red = 0xFFFE;
+  termscreen->termstate.fg_red = 0xfffe;
+  termscreen->termstate.bg_red = 0xfffe;
   termstate                    = &termscreen->termstate;
 
   /* Setup initial RGB colors */
@@ -172,7 +175,7 @@ static int PDC_scr_open_term(int argc, char **argv)
       greylevel                       += (i & COLOR_BLUE)  ? 0x40 : 0;
 
       termstate->greylevel[i]          = greylevel;
-      termstate->greylevel[i+8]        = greylevel | 0x3f;
+      termstate->greylevel[i + 8]      = greylevel | 0x3f;
 #else
       termstate->rgbcolor[i].red       = (i & COLOR_RED)   ? 0xc0 : 0;
       termstate->rgbcolor[i].green     = (i & COLOR_GREEN) ? 0xc0 : 0;
@@ -207,18 +210,20 @@ static int PDC_scr_open_term(int argc, char **argv)
  * Name: PDC_init_pair_term
  *
  * Description:
- *   The core of init_pair().  This does all the work of that function, except
- *   checking for values out of range.  The values passed to this function
- *   should be returned by a call to PDC_pair_content() with the same pair
- *   number.  PDC_transform_line() should use the specified colors when
- *   rendering a chtype with the given pair number.
+ *   The core of init_pair().  This does all the work of that function,
+ *   except checking for values out of range.  The values passed to this
+ *   function should be returned by a call to PDC_pair_content() with the
+ *   same pair number.  PDC_transform_line() should use the specified colors
+ *   when rendering a chtype with the given pair number.
  *
  ****************************************************************************/
 
 #ifdef CONFIG_SYSTEM_TERMCURSES
-static void PDC_init_pair_term(FAR SCREEN *sp, short pair, short fg, short bg)
+static void PDC_init_pair_term(FAR SCREEN *sp, short pair,
+                               short fg, short bg)
 {
-  FAR struct pdc_termscreen_s *termscreen = (FAR struct pdc_termscreen_s *)sp;
+  FAR struct pdc_termscreen_s *termscreen =
+    (FAR struct pdc_termscreen_s *)sp;
 
   termscreen->termstate.colorpair[pair].fg = fg;
   termscreen->termstate.colorpair[pair].bg = bg;
@@ -238,7 +243,8 @@ static void PDC_init_pair_term(FAR SCREEN *sp, short pair, short fg, short bg)
 int PDC_init_color_term(FAR SCREEN *sp, short color, short red, short green,
                         short blue)
 {
-  FAR struct pdc_termscreen_s *termscreen = (FAR struct pdc_termscreen_s *)sp;
+  FAR struct pdc_termscreen_s *termscreen =
+    (FAR struct pdc_termscreen_s *)sp;
   FAR struct pdc_termstate_s  *termstate;
 
   DEBUGASSERT(termscreen != NULL);
@@ -272,7 +278,8 @@ int PDC_init_color_term(FAR SCREEN *sp, short color, short red, short green,
 int PDC_color_content_term(FAR SCREEN *sp, short color,
                            FAR short *red, FAR short *green, FAR short *blue)
 {
-  FAR struct pdc_termscreen_s *termscreen = (FAR struct pdc_termscreen_s *)sp;
+  FAR struct pdc_termscreen_s *termscreen =
+    (FAR struct pdc_termscreen_s *)sp;
   FAR struct pdc_termstate_s  *termstate;
 
   DEBUGASSERT(termscreen != NULL);
@@ -303,9 +310,11 @@ int PDC_color_content_term(FAR SCREEN *sp, short color,
  ****************************************************************************/
 
 #ifdef CONFIG_SYSTEM_TERMCURSES
-static int PDC_pair_content_term(FAR SCREEN *sp, short pair, short *fg, short *bg)
+static int PDC_pair_content_term(FAR SCREEN *sp, short pair,
+                                 FAR short *fg, FAR short *bg)
 {
-  FAR struct pdc_termscreen_s *termscreen = (FAR struct pdc_termscreen_s *)sp;
+  FAR struct pdc_termscreen_s *termscreen =
+    (FAR struct pdc_termscreen_s *)sp;
 
   *fg = termscreen->termstate.colorpair[pair].fg;
   *bg = termscreen->termstate.colorpair[pair].bg;
@@ -442,7 +451,8 @@ int PDC_scr_open(int argc, char **argv)
 
   /* Allocate the global instance of SP */
 
-  fbscreen = (FAR struct pdc_fbscreen_s *)zalloc(sizeof(struct pdc_fbscreen_s));
+  fbscreen =
+    (FAR struct pdc_fbscreen_s *)zalloc(sizeof(struct pdc_fbscreen_s));
   if (fbscreen == NULL)
     {
       PDC_LOG(("ERROR: Failed to allocate SP\n"));
@@ -473,7 +483,7 @@ int PDC_scr_open(int argc, char **argv)
       greylevel                     += (i & COLOR_BLUE)  ? 0x40 : 0;
 
       fbstate->greylevel[i]          = greylevel;
-      fbstate->greylevel[i+8]        = greylevel | 0x3f;
+      fbstate->greylevel[i + 8]      = greylevel | 0x3f;
 #else
       fbstate->rgbcolor[i].red       = (i & COLOR_RED)   ? 0xc0 : 0;
       fbstate->rgbcolor[i].green     = (i & COLOR_GREEN) ? 0xc0 : 0;
@@ -564,8 +574,8 @@ int PDC_scr_open(int argc, char **argv)
    * address mapping to make the memory accessible to the application.
    */
 
-  fbstate->fbmem = mmap(NULL, pinfo.fblen, PROT_READ|PROT_WRITE,
-                        MAP_SHARED|MAP_FILE, fbstate->fbfd, 0);
+  fbstate->fbmem = mmap(NULL, pinfo.fblen, PROT_READ | PROT_WRITE,
+                        MAP_SHARED | MAP_FILE, fbstate->fbfd, 0);
   if (fbstate->fbmem == MAP_FAILED)
     {
       PDC_LOG(("ERROR: ioctl(FBIOGET_PLANEINFO) failed: %d\n", errno));
@@ -704,9 +714,9 @@ int PDC_resize_screen(int nlines, int ncols)
  *
  * Description:
  *   The non-portable functionality of reset_prog_mode() is handled here --
- *   whatever's not done in _restore_mode().  In current ports:  In OS/2, this
- *   sets the keyboard to binary mode; in Win32, it enables or disables the
- *   mouse pointer to match the saved mode; in others it does nothing.
+ *   whatever's not done in _restore_mode().  In current ports:  In OS/2,
+ *   this sets the keyboard to binary mode; in Win32, it enables or disables
+ *   the mouse pointer to match the saved mode; in others it does nothing.
  *
  ****************************************************************************/
 
@@ -763,11 +773,11 @@ void PDC_save_screen_mode(int i)
  * Name: PDC_init_pair
  *
  * Description:
- *   The core of init_pair().  This does all the work of that function, except
- *   checking for values out of range.  The values passed to this function
- *   should be returned by a call to PDC_pair_content() with the same pair
- *   number.  PDC_transform_line() should use the specified colors when
- *   rendering a chtype with the given pair number.
+ *   The core of init_pair().  This does all the work of that function,
+ *   except checking for values out of range.  The values passed to this
+ *   function should be returned by a call to PDC_pair_content() with the
+ *   same pair number.  PDC_transform_line() should use the specified colors
+ *   when rendering a chtype with the given pair number.
  *
  ****************************************************************************/
 
