@@ -58,6 +58,7 @@
 
 #include <net/if.h>
 #include <netinet/in.h>
+#include <nuttx/net/netdev.h>
 #include <nuttx/net/netconfig.h>
 
 /****************************************************************************
@@ -375,6 +376,12 @@ ssize_t netlib_get_route(FAR struct rtentry *rtelist,
                          unsigned int nentries, sa_family_t family);
 #endif
 
+#if defined(CONFIG_NET_IPv4) && defined(CONFIG_NETUTILS_DHCPC)
+/* DHCP */
+
+int netlib_obtain_ipv4addr(FAR const char *ifname);
+#endif
+
 #ifdef CONFIG_NET_ICMPv6_AUTOCONF
 /* ICMPv6 Autoconfiguration */
 
@@ -489,6 +496,11 @@ int netlib_set_ipv6dnsaddr(FAR const struct in6_addr *inaddr);
 
 int netlib_set_mtu(FAR const char *ifname, int mtu);
 
+#if defined(CONFIG_NETDEV_STATISTICS)
+int netlib_getifstatistics(FAR const char *ifname,
+                           FAR struct netdev_statistics_s *stat);
+#endif
+
 #undef EXTERN
 #ifdef __cplusplus
 }
