@@ -53,7 +53,7 @@ struct nxplayer_s
   int             state;                       /* Current player state */
   int             dev_fd;                      /* File descriptor of active device */
   mqd_t           mq;                          /* Message queue for the playthread */
-  char            mqname[16];                  /* Name of our message queue */
+  char            mqname[32];                  /* Name of our message queue */
   pthread_t       play_id;                     /* Thread ID of the playthread */
   int             crefs;                       /* Number of references to the player */
   pthread_mutex_t mutex;                       /* Thread sync mutex */
@@ -506,6 +506,23 @@ int nxplayer_systemreset(FAR struct nxplayer_s *pplayer);
 int nxplayer_parse_mp3(int fd, FAR uint32_t *samplerate,
                        FAR uint8_t *chans, FAR uint8_t *bps);
 
+/****************************************************************************
+ * Name: nxplayer_parse_sbc
+ *
+ *   Performs pre-process when play sbc file.
+ *   Parse samplerate, channels, bps.
+ *
+ * Input Parameters:
+ *   pplayer   - Pointer to the context to initialize
+ *
+ * Returned Value:
+ *   OK if file parsed successfully.
+ *
+ ****************************************************************************/
+
+int nxplayer_parse_sbc(int fd, FAR uint32_t *samplerate,
+                       FAR uint8_t *chans, FAR uint8_t *bps);
+
 /****************************************************************************
  * Name: nxplayer_fill_mp3
  *
