@@ -32,6 +32,8 @@
 #define TOYWASM_USE_SMALL_CELLS
 #define TOYWASM_USE_RESULTTYPE_CELLIDX
 #define TOYWASM_USE_LOCALTYPE_CELLIDX
+#define TOYWASM_ENABLE_HEAP_TRACKING
+#define TOYWASM_ENABLE_HEAP_TRACKING_PEAK
 #define TOYWASM_ENABLE_WRITER
 #define TOYWASM_ENABLE_WASM_SIMD
 #define TOYWASM_ENABLE_WASM_EXCEPTION_HANDLING
