@@ -116,7 +116,7 @@ static int send_getversion_onreset(FAR struct alt1250_s *dev,
   set_container_postproc(container, postproc_ponresetseq,
                          (unsigned long)&reset_arg);
 
-  return altdevice_send_command(dev->altfd, container, usock_result);
+  return altdevice_send_command(dev, dev->altfd, container, usock_result);
 }
 
 /****************************************************************************
@@ -192,6 +192,37 @@ static int recv_atreply_onreset(atreply_parser_t parse,
   return ret;
 }
 
+/****************************************************************************
+ * name: alt1250_disable_scanplan
+ ****************************************************************************/
+
+static int alt1250_disable_scanplan(FAR struct alt1250_s *dev,
+                                    FAR struct alt_container_s *container)
+{
+  int recv_ret;
+  struct atreply_truefalse_s t_or_f;
+
+  t_or_f.target_str = "0";
+  ltesp_send_getscanplan(dev, container);
+  recv_ret = recv_atreply_onreset(check_atreply_truefalse, dev, &t_or_f);
+  if (recv_ret == REP_MODEM_RESET)
+    {
+      return recv_ret;
+    }
+
+  if (!t_or_f.result)
+    {
+      ltesp_send_setscanplan(dev, container, false);
+      recv_ret = recv_atreply_onreset(check_atreply_ok, dev, NULL);
+      if (recv_ret == REP_MODEM_RESET)
+        {
+          return recv_ret;
+        }
+    }
+
+  return recv_ret;
+}
+
 /****************************************************************************
  * name: alt1250_lwm2m_ponreset
  ****************************************************************************/
@@ -319,6 +350,14 @@ static int alt1250_lwm2m_ponreset(FAR struct alt1250_s *dev,
       ret = REP_SEND_ACK;
     }
 
+  /* Make sure SCAN_PALAN_EN is 0 */
+
+  recv_ret = alt1250_disable_scanplan(dev, container);
+  if (recv_ret == REP_MODEM_RESET)
+    {
+      return recv_ret;
+    }
+
   if (ret == REP_SEND_ACK)
     {
       /* Force Reset is needed. */
