@@ -69,7 +69,8 @@
  ****************************************************************************/
 
 int nsh_builtin(FAR struct nsh_vtbl_s *vtbl, FAR const char *cmd,
-                FAR char **argv, FAR const char *redirfile, int oflags)
+                FAR char **argv, FAR const char *redirfile_in,
+                FAR const char *redirfile_out, int oflags)
 {
 #if !defined(CONFIG_NSH_DISABLEBG) && defined(CONFIG_SCHED_CHILD_STATUS)
   struct sigaction act;
@@ -101,7 +102,7 @@ int nsh_builtin(FAR struct nsh_vtbl_s *vtbl, FAR const char *cmd,
    * applications.
    */
 
-  ret = exec_builtin(cmd, argv, redirfile, oflags);
+  ret = exec_builtin(cmd, argv, redirfile_in, redirfile_out, oflags);
   if (ret >= 0)
     {
       /* The application was successfully started with pre-emption disabled.
