@@ -41,18 +41,19 @@
 
 /* Method access macros */
 
-#define nsh_clone(v)           (v)->clone(v)
-#define nsh_release(v)         (v)->release(v)
-#define nsh_write(v,b,n)       (v)->write(v,b,n)
-#define nsh_ioctl(v,c,a)       (v)->ioctl(v,c,a)
-#define nsh_linebuffer(v)      (v)->linebuffer(v)
-#define nsh_redirect(v,f,s)    (v)->redirect(v,f,s)
-#define nsh_undirect(v,s)      (v)->undirect(v,s)
-#define nsh_exit(v,s)          (v)->exit(v,s)
+#define nsh_clone(v)            (v)->clone(v)
+#define nsh_release(v)          (v)->release(v)
+#define nsh_write(v,b,n)        (v)->write(v,b,n)
+#define nsh_read(v,b,n)         (v)->read(v,b,n)
+#define nsh_ioctl(v,c,a)        (v)->ioctl(v,c,a)
+#define nsh_linebuffer(v)       (v)->linebuffer(v)
+#define nsh_redirect(v,fi,fo,s) (v)->redirect(v,fi,fo,s)
+#define nsh_undirect(v,s)       (v)->undirect(v,s)
+#define nsh_exit(v,s)           (v)->exit(v,s)
 
 #ifdef CONFIG_CPP_HAVE_VARARGS
-#  define nsh_error(v, ...)    (v)->error(v, ##__VA_ARGS__)
-#  define nsh_output(v, ...)   (v)->output(v, ##__VA_ARGS__)
+#  define nsh_error(v, ...)     (v)->error(v, ##__VA_ARGS__)
+#  define nsh_output(v, ...)    (v)->output(v, ##__VA_ARGS__)
 #  define nsh_none(v, ...)     \
      do { if (0) nsh_output_none(v, ##__VA_ARGS__); } while (0)
 #else
@@ -85,7 +86,7 @@
 
 #else
 
-#  define INFD(p)      0
+#  define INFD(p)      ((p)->cn_infd)
 
 #endif
 
@@ -113,6 +114,8 @@ struct nsh_vtbl_s
   void (*release)(FAR struct nsh_vtbl_s *vtbl);
   ssize_t (*write)(FAR struct nsh_vtbl_s *vtbl, FAR const void *buffer,
                    size_t nbytes);
+  ssize_t (*read)(FAR struct nsh_vtbl_s *vtbl, FAR void *buffer,
+                   size_t nbytes);
   int (*ioctl)(FAR struct nsh_vtbl_s *vtbl, int cmd, unsigned long arg);
 #ifndef CONFIG_NSH_DISABLE_ERROR_PRINT
   int (*error)(FAR struct nsh_vtbl_s *vtbl, FAR const char *fmt, ...)
@@ -121,7 +124,8 @@ struct nsh_vtbl_s
   int (*output)(FAR struct nsh_vtbl_s *vtbl, FAR const char *fmt, ...)
       printf_like(2, 3);
   FAR char *(*linebuffer)(FAR struct nsh_vtbl_s *vtbl);
-  void (*redirect)(FAR struct nsh_vtbl_s *vtbl, int fd, FAR uint8_t *save);
+  void (*redirect)(FAR struct nsh_vtbl_s *vtbl, int fd_in, int fd_out,
+                   FAR uint8_t *save);
   void (*undirect)(FAR struct nsh_vtbl_s *vtbl, FAR uint8_t *save);
   void (*exit)(FAR struct nsh_vtbl_s *vtbl, int status) noreturn_function;
 
@@ -163,6 +167,7 @@ struct console_stdio_s
 #ifdef CONFIG_NSH_ALTCONDEV
   int   cn_confd;     /* Console I/O file descriptor */
 #endif
+  int   cn_infd;      /* Input file descriptor (possibly redirected) */
   int   cn_outfd;     /* Output file descriptor (possibly redirected) */
   int   cn_errfd;     /* Error Output file descriptor (possibly redirected) */
 
