@@ -163,8 +163,8 @@ static int cp_handler(FAR struct nsh_vtbl_s *vtbl, FAR const char *srcpath,
 
   for (; ; )
     {
-      int nbytesread;
-      int nbyteswritten;
+      ssize_t nbyteswritten;
+      ssize_t nbytesread;
       FAR char *iobuffer = vtbl->iobuffer;
 
       nbytesread = read(rdfd, iobuffer, IOBUFFERSIZE);
@@ -791,6 +791,25 @@ int cmd_cat(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
         }
     }
 
+  if (argc == 1)
+    {
+      char *buf = malloc(BUFSIZ);
+
+      /* Dump from input */
+
+      while (true)
+        {
+          ssize_t n = nsh_read(vtbl, buf, BUFSIZ);
+
+          if (n == 0)
+            break;
+
+          nsh_write(vtbl, buf, n);
+        }
+
+      free(buf);
+    }
+
   return ret;
 }
 #endif
@@ -2101,21 +2120,20 @@ int cmd_readlink(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
 #ifdef NSH_HAVE_DIROPTS
 #ifndef CONFIG_NSH_DISABLE_RM
 
-static int unlink_recursive(FAR char *path)
+static int unlink_recursive(FAR char *path, FAR struct stat *stat)
 {
   struct dirent *d;
-  struct stat stat;
   size_t len;
   int ret;
   DIR *dp;
 
-  ret = lstat(path, &stat);
+  ret = lstat(path, stat);
   if (ret < 0)
     {
       return ret;
     }
 
-  if (!S_ISDIR(stat.st_mode))
+  if (!S_ISDIR(stat->st_mode))
     {
       return unlink(path);
     }
@@ -2140,7 +2158,7 @@ static int unlink_recursive(FAR char *path)
         }
 
       snprintf(&path[len], PATH_MAX - len, "/%s", d->d_name);
-      ret = unlink_recursive(path);
+      ret = unlink_recursive(path, stat);
       if (ret < 0)
         {
           closedir(dp);
@@ -2160,31 +2178,65 @@ static int unlink_recursive(FAR char *path)
 
 int cmd_rm(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
 {
-  bool recursive = (strcmp(argv[1], "-r") == 0);
+  bool recursive = false;
+  bool force = false;
   FAR char *fullpath;
   char buf[PATH_MAX];
+  struct stat stat;
   int ret = ERROR;
+  int c;
 
-  if (recursive && argc == 2)
+  while ((c = getopt(argc, argv, "rf")) != ERROR)
     {
-      nsh_error(vtbl, g_fmtargrequired, argv[0]);
+      switch (c)
+        {
+          case 'r':
+            recursive = true;
+            break;
+          case 'f':
+            force = true;
+            break;
+          case '?':
+            nsh_output(vtbl, "Unknown option 0x%x\n", optopt);
+            return ret;
+          default:
+            nsh_error(vtbl, g_fmtargrequired, argv[0]);
+            return ret;
+        }
+    }
+
+  if (optind >= argc)
+    {
+      if (force)
+        {
+          ret = OK;
+        }
+      else
+        {
+          nsh_error(vtbl, g_fmtargrequired, argv[0]);
+        }
+
       return ret;
     }
 
-  fullpath = nsh_getfullpath(vtbl, recursive ? argv[2] : argv[1]);
-
+  fullpath = nsh_getfullpath(vtbl, argv[optind]);
   if (fullpath != NULL)
     {
       if (recursive)
         {
           strlcpy(buf, fullpath, PATH_MAX);
-          ret = unlink_recursive(buf);
+          ret = unlink_recursive(buf, &stat);
         }
       else
         {
           ret = unlink(fullpath);
         }
 
+      if (force && errno == ENOENT)
+        {
+          ret = 0;
+        }
+
       if (ret < 0)
         {
           nsh_error(vtbl, g_fmtcmdfailed, argv[0], "unlink", NSH_ERRNO);
