@@ -434,7 +434,8 @@ int cmd_addroute(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
 
   /* Then add the route */
 
-  ret = addroute(sockfd, &target.ipx, &netmask.ipx, &router.ipx);
+  ret = addroute(sockfd, &target.ipx, &netmask.ipx,
+                 &router.ipx, sizeof(router));
   if (ret < 0)
     {
       nsh_error(vtbl, g_fmtcmdfailed, argv[0], "addroute", NSH_ERRNO);
@@ -676,7 +677,7 @@ int cmd_delroute(FAR struct nsh_vtbl_s *vtbl, int argc, FAR char **argv)
 
   /* Then delete the route */
 
-  ret = delroute(sockfd, &target.ipx, &netmask.ipx);
+  ret = delroute(sockfd, &target.ipx, &netmask.ipx, sizeof(target));
   if (ret < 0)
     {
       nsh_error(vtbl, g_fmtcmdfailed, argv[0], "delroute", NSH_ERRNO);
