@@ -35,6 +35,14 @@
 
 #ifndef CONFIG_NSH_DISABLESCRIPT
 
+/****************************************************************************
+ * Private Data
+ ****************************************************************************/
+
+#ifdef CONFIG_ETC_ROMFS
+static bool g_nsh_script_initialized;
+#endif
+
 /****************************************************************************
  * Private Functions
  ****************************************************************************/
@@ -54,7 +62,7 @@ static int nsh_script_redirect(FAR struct nsh_vtbl_s *vtbl,
       fd = open(CONFIG_NSH_SCRIPT_REDIRECT_PATH, 0666);
       if (fd > 0)
         {
-          nsh_redirect(vtbl, fd, save);
+          nsh_redirect(vtbl, 0, fd, save);
         }
     }
 
@@ -111,7 +119,7 @@ int nsh_script(FAR struct nsh_vtbl_s *vtbl, FAR const FAR char *cmd,
 
       /* Open the file containing the script */
 
-      vtbl->np.np_fd = open(fullpath, O_RDOK);
+      vtbl->np.np_fd = open(fullpath, O_RDOK | O_CLOEXEC);
       if (vtbl->np.np_fd < 0)
         {
           if (log)
@@ -227,15 +235,14 @@ int nsh_sysinitscript(FAR struct nsh_vtbl_s *vtbl)
 #ifdef CONFIG_ETC_ROMFS
 int nsh_initscript(FAR struct nsh_vtbl_s *vtbl)
 {
-  static bool initialized;
   bool already;
   int ret = OK;
 
-  /* Atomic test and set of the initialized flag */
+  /* Atomic test and set of the g_nsh_script_initialized flag */
 
   sched_lock();
-  already     = initialized;
-  initialized = true;
+  already                  = g_nsh_script_initialized;
+  g_nsh_script_initialized = true;
   sched_unlock();
 
   /* If we have not already executed the init script, then do so now */
