@@ -23,6 +23,7 @@
  ****************************************************************************/
 
 #include <nuttx/config.h>
+#include <nuttx/mtd/mtd.h>
 #include <nuttx/version.h>
 
 #include <errno.h>
@@ -32,12 +33,16 @@
 #include <stdint.h>
 #include <stdio.h>
 #include <string.h>
+#include <syslog.h>
+#include <unistd.h>
 
 #include <sys/boardctl.h>
 #include <sys/ioctl.h>
 #include <sys/param.h>
+#include <sys/stat.h>
 #include <sys/statfs.h>
 #include <sys/types.h>
+#include <sys/poll.h>
 
 /****************************************************************************
  * Pre-processor Definitions
@@ -46,8 +51,10 @@
 #define FASTBOOT_USBDEV             "/dev/fastboot"
 #define FASTBOOT_BLKDEV             "/dev/%s"
 
-#define FASTBOOT_EP_BULKIN_IDX      0
-#define FASTBOOT_EP_BULKOUT_IDX     1
+#define FASTBOOT_EP_BULKIN_IDX      1
+#define FASTBOOT_EP_BULKOUT_IDX     2
+#define FASTBOOT_EP_RETRY_TIMES     100
+#define FASTBOOT_EP_RETRY_DELAY_MS  10
 
 #define FASTBOOT_MSG_LEN            64
 
@@ -65,6 +72,9 @@
                                      ((uint32_t)(p)[1] << 8) | \
                                      (uint32_t)(p)[0])
 
+#define fb_info(...)                syslog(LOG_INFO, ##__VA_ARGS__);
+#define fb_err(...)                 syslog(LOG_ERR, ##__VA_ARGS__);
+
 /****************************************************************************
  * Private types
  ****************************************************************************/
@@ -100,16 +110,39 @@ struct fastboot_chunk_header_s
   uint32_t total_sz;        /* in bytes of chunk input file including chunk header and data */
 };
 
+struct fastboot_mem_s
+{
+  FAR void *addr;
+};
+
+struct fastboot_file_s
+{
+  char path[PATH_MAX];
+  off_t offset;
+};
+
 struct fastboot_ctx_s
 {
   int usbdev_in;
   int usbdev_out;
+  int flash_fd;
   size_t download_max;
   size_t download_size;
   size_t download_offset;
   size_t total_imgsize;
+  int wait_ms;
   FAR void *download_buffer;
   FAR struct fastboot_var_s *varlist;
+  CODE int (*upload_func)(FAR struct fastboot_ctx_s *context);
+  struct
+    {
+      size_t size;
+      union
+        {
+          struct fastboot_mem_s mem;
+          struct fastboot_file_s file;
+        } u;
+    } upload_param;
 };
 
 struct fastboot_cmd_s
@@ -135,6 +168,15 @@ static void fastboot_reboot(FAR struct fastboot_ctx_s *context,
                             FAR const char *arg);
 static void fastboot_reboot_bootloader(FAR struct fastboot_ctx_s *context,
                                        FAR const char *arg);
+static void fastboot_oem(FAR struct fastboot_ctx_s *context,
+                         FAR const char *arg);
+static void fastboot_upload(FAR struct fastboot_ctx_s *context,
+                            FAR const char *arg);
+
+static void fastboot_memdump(FAR struct fastboot_ctx_s *context,
+                             FAR const char *arg);
+static void fastboot_filedump(FAR struct fastboot_ctx_s *context,
+                              FAR const char *arg);
 
 /****************************************************************************
  * Private Data
@@ -147,7 +189,15 @@ static const struct fastboot_cmd_s g_fast_cmd[] =
   { "erase:",             fastboot_erase            },
   { "flash:",             fastboot_flash            },
   { "reboot-bootloader",  fastboot_reboot_bootloader},
-  { "reboot",             fastboot_reboot           }
+  { "reboot",             fastboot_reboot           },
+  { "oem",                fastboot_oem              },
+  { "upload",             fastboot_upload           }
+};
+
+static const struct fastboot_cmd_s g_oem_cmd[] =
+{
+  { "filedump",           fastboot_filedump         },
+  { "memdump",            fastboot_memdump          }
 };
 
 /****************************************************************************
@@ -220,10 +270,10 @@ static void fastboot_okay(FAR struct fastboot_ctx_s *context,
 
 static int fastboot_flash_open(FAR const char *name)
 {
-  int fd = open(name, O_RDWR);
+  int fd = open(name, O_RDWR | O_CLOEXEC);
   if (fd < 0)
     {
-      printf("Open %s error\n", name);
+      fb_err("Open %s error\n", name);
       return -errno;
     }
 
@@ -234,6 +284,7 @@ static void fastboot_flash_close(int fd)
 {
   if (fd >= 0)
     {
+      fsync(fd);
       close(fd);
     }
 }
@@ -247,14 +298,14 @@ static int fastboot_flash_write(int fd, off_t offset,
   offset = lseek(fd, offset, SEEK_SET);
   if (offset < 0)
     {
-      printf("Seek error:%d\n", errno);
+      fb_err("Seek error:%d\n", errno);
       return -errno;
     }
 
   ret = fastboot_write(fd, data, size);
   if (ret < 0)
     {
-      printf("Flash write error:%d\n", -ret);
+      fb_err("Flash write error:%d\n", -ret);
     }
 
   return ret;
@@ -272,7 +323,7 @@ static int ffastboot_flash_fill(int fd, off_t offset,
   buffer = malloc(blk_sz);
   if (buffer == NULL)
     {
-      printf("Flash bwrite malloc fail\n");
+      fb_err("Flash bwrite malloc fail\n");
       return -ENOMEM;
     }
 
@@ -296,13 +347,22 @@ out:
 
 static int fastboot_flash_erase(int fd)
 {
-  return OK;
+  int ret;
+
+  ret = ioctl(fd, MTDIOC_BULKERASE, 0);
+  if (ret < 0)
+    {
+      fb_err("Erase device failed\n");
+    }
+
+  return ret < 0 ? -errno : ret;
 }
 
 static int
 fastboot_flash_program(FAR struct fastboot_ctx_s *context, int fd)
 {
   FAR char *chunk_ptr = context->download_buffer;
+  FAR char *end_ptr = chunk_ptr + context->download_size;
   FAR struct fastboot_sparse_header_s *sparse;
   uint32_t chunk_num;
   int ret = OK;
@@ -312,8 +372,9 @@ fastboot_flash_program(FAR struct fastboot_ctx_s *context, int fd)
   sparse = (FAR struct fastboot_sparse_header_s *)chunk_ptr;
   if (sparse->magic != FASTBOOT_SPARSE_MAGIC)
     {
-      return fastboot_flash_write(fd, 0, context->download_buffer,
-                                  context->download_size);
+      ret = fastboot_flash_write(fd, 0, context->download_buffer,
+                                 context->download_size);
+      goto end;
     }
 
   if (context->total_imgsize == 0)
@@ -322,10 +383,9 @@ fastboot_flash_program(FAR struct fastboot_ctx_s *context, int fd)
     }
 
   chunk_num = sparse->total_chunks;
-
   chunk_ptr += FASTBOOT_SPARSE_HEADER;
 
-  while (chunk_num--)
+  while (chunk_ptr < end_ptr && chunk_num--)
     {
       FAR struct fastboot_chunk_header_s *chunk =
         (FAR struct fastboot_chunk_header_s *)chunk_ptr;
@@ -341,7 +401,7 @@ fastboot_flash_program(FAR struct fastboot_ctx_s *context, int fd)
                                          chunk_ptr, chunk_size);
               if (ret < 0)
                 {
-                  return ret;
+                  goto end;
                 }
 
               context->download_offset += chunk_size;
@@ -357,7 +417,7 @@ fastboot_flash_program(FAR struct fastboot_ctx_s *context, int fd)
                                          chunk->chunk_sz);
               if (ret < 0)
                 {
-                  return ret;
+                  goto end;
                 }
 
               context->download_offset += chunk_size;
@@ -368,17 +428,19 @@ fastboot_flash_program(FAR struct fastboot_ctx_s *context, int fd)
           case FASTBOOT_CHUNK_CRC32:
             break;
           default:
-            printf("Error chunk type:%d, skip\n", chunk->chunk_type);
+            fb_err("Error chunk type:%d, skip\n", chunk->chunk_type);
             break;
         }
     }
 
-  if (context->download_offset >= context->total_imgsize)
+  if (context->download_offset < context->total_imgsize)
     {
-      context->total_imgsize = 0;
-      context->download_offset = 0;
+      return 1;
     }
 
+end:
+  context->total_imgsize = 0;
+  context->download_offset = 0;
   return ret;
 }
 
@@ -386,18 +448,22 @@ static void fastboot_flash(FAR struct fastboot_ctx_s *context,
                            FAR const char *arg)
 {
   char blkdev[PATH_MAX];
-  int fd;
+  int ret;
 
   snprintf(blkdev, PATH_MAX, FASTBOOT_BLKDEV, arg);
 
-  fd = fastboot_flash_open(blkdev);
-  if (fd < 0)
+  if (context->flash_fd < 0)
     {
-      fastboot_fail(context, "Flash open failure");
-      return;
+      context->flash_fd = fastboot_flash_open(blkdev);
+      if (context->flash_fd < 0)
+        {
+          fastboot_fail(context, "Flash open failure");
+          return;
+        }
     }
 
-  if (fastboot_flash_program(context, fd) < 0)
+  ret = fastboot_flash_program(context, context->flash_fd);
+  if (ret < 0)
     {
       fastboot_fail(context, "Image flash failure");
     }
@@ -406,17 +472,22 @@ static void fastboot_flash(FAR struct fastboot_ctx_s *context,
       fastboot_okay(context, "");
     }
 
-  fastboot_flash_close(fd);
+  if (ret <= 0)
+    {
+      fastboot_flash_close(context->flash_fd);
+      context->flash_fd = -1;
+    }
 }
 
 static void fastboot_erase(FAR struct fastboot_ctx_s *context,
                            FAR const char *arg)
 {
   char blkdev[PATH_MAX];
+  int ret;
   int fd;
 
   snprintf(blkdev, PATH_MAX, FASTBOOT_BLKDEV, arg);
-  printf("Erase %s\n", blkdev);
+  fb_info("Erase %s\n", blkdev);
 
   fd = fastboot_flash_open(blkdev);
   if (fd < 0)
@@ -425,7 +496,32 @@ static void fastboot_erase(FAR struct fastboot_ctx_s *context,
       return;
     }
 
-  if (fastboot_flash_erase(fd) < 0)
+  ret = fastboot_flash_erase(fd);
+  if (ret == -ENOTTY)
+    {
+      struct stat sb;
+
+      ret = fstat(fd, &sb);
+      if (ret >= 0)
+        {
+          memset(context->download_buffer, 0xff, context->download_max);
+
+          while (sb.st_size > 0)
+            {
+              size_t len = MIN(sb.st_size, context->download_max);
+
+              ret = fastboot_write(fd, context->download_buffer, len);
+              if (ret < 0)
+                {
+                  break;
+                }
+
+              sb.st_size -= len;
+            }
+        }
+    }
+
+  if (ret < 0)
     {
       fastboot_fail(context, "Flash erase failure");
     }
@@ -456,11 +552,12 @@ static void fastboot_download(FAR struct fastboot_ctx_s *context,
   ret = fastboot_write(context->usbdev_out, response, strlen(response));
   if (ret < 0)
     {
-      printf("Reponse error [%d]\n", -ret);
+      fb_err("Reponse error [%d]\n", -ret);
       return;
     }
 
   download = context->download_buffer;
+  context->download_size = len;
 
   while (len > 0)
     {
@@ -468,7 +565,8 @@ static void fastboot_download(FAR struct fastboot_ctx_s *context,
                                 download, len);
       if (r < 0)
         {
-          printf("fastboot_download usb read error\n");
+          context->download_size = 0;
+          fb_err("fastboot_download usb read error\n");
           return;
         }
 
@@ -476,7 +574,6 @@ static void fastboot_download(FAR struct fastboot_ctx_s *context,
       download += r;
     }
 
-  context->download_size = len;
   fastboot_okay(context, "");
 }
 
@@ -511,7 +608,10 @@ static void fastboot_reboot(FAR struct fastboot_ctx_s *context,
                             FAR const char *arg)
 {
 #ifdef CONFIG_BOARDCTL_RESET
+  fastboot_okay(context, "");
   boardctl(BOARDIOC_RESET, BOARDIOC_SOFTRESETCAUSE_USER_REBOOT);
+#else
+  fastboot_fail(context, "Operation not supported");
 #endif
 }
 
@@ -519,15 +619,235 @@ static void fastboot_reboot_bootloader(FAR struct fastboot_ctx_s *context,
                                        FAR const char *arg)
 {
 #ifdef CONFIG_BOARDCTL_RESET
+  fastboot_okay(context, "");
   boardctl(BOARDIOC_RESET, BOARDIOC_SOFTRESETCAUSE_ENTER_BOOTLOADER);
+#else
+  fastboot_fail(context, "Operation not supported");
 #endif
 }
 
+static int fastboot_memdump_upload(FAR struct fastboot_ctx_s *context)
+{
+  return fastboot_write(context->usbdev_out,
+                        context->upload_param.u.mem.addr,
+                        context->upload_param.size);
+}
+
+/* Usage(host):
+ *   fastboot oem memdump <addr> <size>
+ *
+ * Example
+ *   fastboot oem memdump 0x44000000 0xb6c00
+ *   fastboot get_staged mem_44000000_440b6c00.bin
+ */
+
+static void fastboot_memdump(FAR struct fastboot_ctx_s *context,
+                             FAR const char *arg)
+{
+  if (!arg ||
+      sscanf(arg, "%p %zx",
+             &context->upload_param.u.mem.addr,
+             &context->upload_param.size) != 2)
+    {
+      fastboot_fail(context, "Invalid argument");
+      return;
+    }
+
+  fb_info("Memdump Addr: %p, Size: 0x%zx\n",
+          context->upload_param.u.mem.addr,
+          context->upload_param.size);
+  context->upload_func = fastboot_memdump_upload;
+  fastboot_okay(context, "");
+}
+
+static int fastboot_filedump_upload(FAR struct fastboot_ctx_s *context)
+{
+  size_t size = context->upload_param.size;
+  int fd;
+
+  fd = open(context->upload_param.u.file.path, O_RDONLY | O_CLOEXEC);
+  if (fd < 0)
+    {
+      fb_err("No such file or directory %d\n", errno);
+      return -errno;
+    }
+
+  if (context->upload_param.u.file.offset &&
+      lseek(fd, context->upload_param.u.file.offset,
+            context->upload_param.u.file.offset > 0 ? SEEK_SET :
+                                                      SEEK_END) < 0)
+    {
+      fb_err("Invalid argument, offset: %" PRIdOFF "\n",
+             context->upload_param.u.file.offset);
+      close(fd);
+      return -errno;
+    }
+
+  while (size > 0)
+    {
+      ssize_t nread = fastboot_read(fd, context->download_buffer,
+                                    MIN(size, context->download_max));
+      if (nread == 0)
+        {
+          break;
+        }
+      else if (nread < 0 ||
+               fastboot_write(context->usbdev_out,
+                              context->download_buffer,
+                              nread) < 0)
+        {
+          fb_err("Upload failed (%zu bytes left)\n", size);
+          close(fd);
+          return -errno;
+        }
+
+      size -= nread;
+    }
+
+  close(fd);
+  return 0;
+}
+
+/* Usage(host):
+ *   fastboot oem filedump <PATH> [<offset> <size>]
+ *
+ * Example
+ *   a. Upload the entire file:
+ *      fastboot oem filedump /dev/bootloader
+ *      fastboot get_staged bl_all.bin
+ *
+ *   b. Upload 4096 bytes of /dev/mem from offset 2048:
+ *      fastboot oem filedump /dev/mem 2048 4096
+ *      fastboot get_staged bl_2048_6144.bin
+ *
+ *   c. Get 2048 bytes from offset -1044480
+ *      fastboot oem "filedump /dev/bootloader -1044480 2048"
+ *      fastboot get_staged bl_l1044480_l1042432.txt
+ */
+
+static void fastboot_filedump(FAR struct fastboot_ctx_s *context,
+                              FAR const char *arg)
+{
+  struct stat sb;
+  int ret;
+
+  if (!arg)
+    {
+      fastboot_fail(context, "Invalid argument");
+      return;
+    }
+
+  ret = sscanf(arg, "%s %" PRIdOFF " %zu",
+               context->upload_param.u.file.path,
+               &context->upload_param.u.file.offset,
+               &context->upload_param.size);
+  if (ret != 1 && ret != 3)
+    {
+      fastboot_fail(context, "Failed to parse arguments");
+      return;
+    }
+  else if (ret == 1)
+    {
+      ret = stat(context->upload_param.u.file.path, &sb);
+      if (ret < 0)
+        {
+          fastboot_fail(context, "No such file or directory");
+          return;
+        }
+
+      context->upload_param.size = sb.st_size;
+      context->upload_param.u.file.offset = 0;
+    }
+
+  fb_info("Filedump Path: %s, Offset: %" PRIdOFF ", Size: %zu\n",
+          context->upload_param.u.file.path,
+          context->upload_param.u.file.offset,
+          context->upload_param.size);
+  context->upload_func = fastboot_filedump_upload;
+  fastboot_okay(context, "");
+}
+
+static void fastboot_upload(FAR struct fastboot_ctx_s *context,
+                            FAR const char *arg)
+{
+  char response[FASTBOOT_MSG_LEN];
+  int ret;
+
+  if (!context->upload_param.size || !context->upload_func)
+    {
+      fastboot_fail(context, "No data staged by the last command");
+      return;
+    }
+
+  snprintf(response, FASTBOOT_MSG_LEN, "DATA%08zx",
+           context->upload_param.size);
+
+  ret = fastboot_write(context->usbdev_out, response, strlen(response));
+  if (ret < 0)
+    {
+      fb_err("Reponse error [%d]\n", -ret);
+      goto done;
+    }
+
+  ret = context->upload_func(context);
+  if (ret < 0)
+    {
+      fb_err("Upload failed, [%d]\n", -ret);
+      fastboot_fail(context, "Upload failed");
+    }
+  else
+    {
+      fastboot_okay(context, "");
+    }
+
+done:
+  context->upload_param.size = 0;
+  context->upload_func = NULL;
+}
+
+static void fastboot_oem(FAR struct fastboot_ctx_s *context,
+                         FAR const char *arg)
+{
+  size_t ncmds = nitems(g_oem_cmd);
+  size_t index;
+
+  arg++;
+
+  for (index = 0; index < ncmds; index++)
+    {
+      size_t len = strlen(g_oem_cmd[index].prefix);
+      if (memcmp(arg, g_oem_cmd[index].prefix, len) == 0)
+        {
+          arg += len;
+          g_oem_cmd[index].handle(context, *arg == ' ' ? ++arg : NULL);
+          break;
+        }
+    }
+
+  if (index == ncmds)
+    {
+      fastboot_fail(context, "Unknown command");
+    }
+}
+
 static void fastboot_command_loop(FAR struct fastboot_ctx_s *context)
 {
+  if (context->wait_ms > 0)
+    {
+      struct pollfd fds[1];
+
+      fds[0].fd = context->usbdev_in;
+      fds[0].events = POLLIN;
+
+      if (poll(fds, 1, context->wait_ms) <= 0)
+        {
+          return;
+        }
+    }
+
   while (1)
     {
-      char buffer[FASTBOOT_MSG_LEN];
+      char buffer[FASTBOOT_MSG_LEN + 1];
       size_t ncmds = nitems(g_fast_cmd);
       size_t index;
 
@@ -535,7 +855,7 @@ static void fastboot_command_loop(FAR struct fastboot_ctx_s *context)
                                 buffer, FASTBOOT_MSG_LEN);
       if (r < 0)
         {
-          printf("USB read error\n");
+          fb_err("USB read error\n");
           break;
         }
 
@@ -565,14 +885,17 @@ static void fastboot_publish(FAR struct fastboot_ctx_s *context,
   FAR struct fastboot_var_s *var;
 
   var = malloc(sizeof(*var));
-  if (var)
+  if (var == NULL)
     {
-      var->name = name;
-      var->string = string;
-      var->data = data;
-      var->next = context->varlist;
-      context->varlist = var;
+      fb_err("ERROR: Could not allocate the memory.\n");
+      return;
     }
+
+  var->name = name;
+  var->string = string;
+  var->data = data;
+  var->next = context->varlist;
+  context->varlist = var;
 }
 
 static void fastboot_create_publish(FAR struct fastboot_ctx_s *context)
@@ -597,48 +920,122 @@ static void fastboot_free_publish(FAR struct fastboot_ctx_s *context)
     }
 }
 
+static int fastboot_open_usb(int index, int flags)
+{
+  int try = FASTBOOT_EP_RETRY_TIMES;
+  char usbdev[32];
+  int ret;
+
+  snprintf(usbdev, sizeof(usbdev),
+           "%s/ep%d", FASTBOOT_USBDEV, index);
+  do
+    {
+      ret = open(usbdev, flags);
+      if (ret >= 0)
+        {
+          return ret;
+        }
+
+      usleep(FASTBOOT_EP_RETRY_DELAY_MS * 1000);
+    }
+  while (try--);
+
+  fb_err("open [%s] error %d\n", usbdev, errno);
+
+  return -errno;
+}
+
 /****************************************************************************
  * Public Functions
  ****************************************************************************/
 
 int main(int argc, FAR char **argv)
 {
-  FAR struct fastboot_ctx_s context;
+  struct fastboot_ctx_s context;
   FAR void *buffer = NULL;
-  char usbdev[32];
   int ret = OK;
 
+#ifdef CONFIG_FASTBOOTD_USB_BOARDCTL
+  struct boardioc_usbdev_ctrl_s ctrl;
+#  ifdef CONFIG_USBDEV_COMPOSITE
+    uint8_t dev = BOARDIOC_USBDEV_COMPOSITE;
+#  else
+    uint8_t dev = BOARDIOC_USBDEV_FASTBOOT;
+#  endif
+  FAR void *handle;
+
+  ctrl.usbdev   = dev;
+  ctrl.action   = BOARDIOC_USBDEV_INITIALIZE;
+  ctrl.instance = 0;
+  ctrl.config   = 0;
+  ctrl.handle   = NULL;
+
+  ret = boardctl(BOARDIOC_USBDEV_CONTROL, (uintptr_t)&ctrl);
+  if (ret < 0)
+    {
+      fb_err("boardctl(BOARDIOC_USBDEV_CONTROL) failed: %d\n", ret);
+      return ret;
+    }
+
+  ctrl.usbdev   = dev;
+  ctrl.action   = BOARDIOC_USBDEV_CONNECT;
+  ctrl.instance = 0;
+  ctrl.config   = 0;
+  ctrl.handle   = &handle;
+
+  ret = boardctl(BOARDIOC_USBDEV_CONTROL, (uintptr_t)&ctrl);
+  if (ret < 0)
+    {
+      fb_err("boardctl(BOARDIOC_USBDEV_CONTROL) failed: %d\n", ret);
+      return ret;
+    }
+#endif /* FASTBOOTD_USB_BOARDCTL */
+
+  if (argc > 1)
+    {
+      if (strcmp(argv[1], "-h") == 0)
+        {
+          fb_err("Usage: fastbootd [wait_ms]\n");
+          return 0;
+        }
+
+      context.wait_ms = atoi(argv[1]);
+    }
+  else
+    {
+      context.wait_ms = 0;
+    }
+
   buffer = malloc(CONFIG_SYSTEM_FASTBOOTD_DOWNLOAD_MAX);
   if (buffer == NULL)
     {
-      printf("ERROR: Could not allocate the memory.\n");
+      fb_err("ERROR: Could not allocate the memory.\n");
       return -ENOMEM;
     }
 
-  snprintf(usbdev, sizeof(usbdev), "%s/ep%d",
-           FASTBOOT_USBDEV, FASTBOOT_EP_BULKOUT_IDX + 1);
-  context.usbdev_in = open(usbdev, O_RDONLY);
+  context.usbdev_in =
+      fastboot_open_usb(FASTBOOT_EP_BULKOUT_IDX, O_RDONLY | O_CLOEXEC);
   if (context.usbdev_in < 0)
     {
-      printf("open [%s] error\n", usbdev);
       ret = -errno;
       goto err_with_mem;
     }
 
-  snprintf(usbdev, sizeof(usbdev), "%s/ep%d",
-           FASTBOOT_USBDEV, FASTBOOT_EP_BULKIN_IDX + 1);
-  context.usbdev_out = open(usbdev, O_WRONLY);
+  context.usbdev_out =
+      fastboot_open_usb(FASTBOOT_EP_BULKIN_IDX, O_WRONLY | O_CLOEXEC);
   if (context.usbdev_out < 0)
     {
-      printf("open [%s] error\n", usbdev);
       ret = -errno;
       goto err_with_in;
     }
 
+  context.varlist         = NULL;
+  context.flash_fd        = -1;
   context.download_buffer = buffer;
   context.download_size   = 0;
   context.download_offset = 0;
   context.download_max    = CONFIG_SYSTEM_FASTBOOTD_DOWNLOAD_MAX;
+  context.total_imgsize   = 0;
 
   fastboot_create_publish(&context);
   fastboot_command_loop(&context);
