@@ -25,15 +25,23 @@
 #include <errno.h>
 #include <math.h>
 #include <poll.h>
+#include <inttypes.h>
 #include <string.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <sys/param.h>
+#include <sched.h>
+#include <pthread.h>
 
 #include "utility.h"
 
+/****************************************************************************
+ * Pre-processor Definitions
+ ****************************************************************************/
+
+#define nitems(_a) (sizeof(_a) / sizeof(0[(_a)]))
+
 /****************************************************************************
  * Private Data
  ****************************************************************************/
@@ -48,7 +56,7 @@ static int           g_pubsubtest_res;
  * Private Functions
  ****************************************************************************/
 
-static int pubsubtest_thread_entry(int argc, FAR char *argv[])
+static FAR void *pubsubtest_thread_entry(FAR void *arg)
 {
   /* poll on test topic and output latency */
 
@@ -68,10 +76,11 @@ static int pubsubtest_thread_entry(int argc, FAR char *argv[])
   unsigned timing_max = 0;
   unsigned i;
 
+  (void)arg;
   timings = malloc(MAX_RUNS * sizeof(unsigned));
   if (timings == NULL)
     {
-      return -ENOMEM;
+      return NULL;
     }
 
   /* clear all ready flags */
@@ -117,7 +126,7 @@ static int pubsubtest_thread_entry(int argc, FAR char *argv[])
 
       if (pret < 0)
         {
-          snerr("poll error %d, %d", pret, errno);
+          printf("poll error %d, %d", pret, errno);
           continue;
         }
     }
@@ -130,14 +139,14 @@ static int pubsubtest_thread_entry(int argc, FAR char *argv[])
       FAR FILE *f;
 
       snprintf(fname, sizeof(fname),
-               CONFIG_UORB_SRORAGE_DIR"/uorb_timings%u.txt", timingsgroup);
+               UORB_STORAGE_DIR"/uorb_timings%u.txt", timingsgroup);
 
       f = fopen(fname, "w");
       if (f == NULL)
         {
-          snerr("Error opening file!");
+          printf("Error opening file!");
           free(timings);
-          return ERROR;
+          return NULL;
         }
 
       for (i = 0; i < MAX_RUNS; i++)
@@ -177,14 +186,15 @@ static int pubsubtest_thread_entry(int argc, FAR char *argv[])
     }
 
   free(timings);
-  return g_pubsubtest_res;
+  return NULL;
 }
 
 static int latency_test(bool print)
 {
   struct orb_test_medium_s sample;
-  int pubsub_task;
+  pthread_t pubsub_task;
   int instance = 0;
+  int ret = ERROR;
   int fd;
 
   test_note("---------------- LATENCY TEST ------------------");
@@ -202,11 +212,11 @@ static int latency_test(bool print)
   g_pubsubtest_print  = print;
   g_pubsubtest_passed = false;
 
-  pubsub_task = task_create("uorb_latency",
-                            SCHED_PRIORITY_DEFAULT,
-                            CONFIG_UORB_STACKSIZE,
-                            pubsubtest_thread_entry,
-                            NULL);
+  if (pthread_create(&pubsub_task, NULL, pubsubtest_thread_entry, NULL) < 0)
+    {
+      test_fail("failed launching task");
+      goto out;
+    }
 
   /* give the test task some data */
 
@@ -216,7 +226,8 @@ static int latency_test(bool print)
       sample.timestamp = orb_absolute_time();
       if (OK != orb_publish(ORB_ID(orb_test_medium), fd, &sample))
         {
-          return test_fail("mult. pub0 timing fail");
+          test_fail("mult. pub0 timing fail");
+          goto out;
         }
 
       usleep(1000); /* simulate >800 Hz system operation */
@@ -224,11 +235,15 @@ static int latency_test(bool print)
 
   if (pubsub_task < 0)
     {
-      return test_fail("failed launching task");
+      test_fail("failed launching task");
+      goto out;
     }
 
+  ret = OK;
+
+out:
   orb_unadvertise(fd);
-  return g_pubsubtest_res;
+  return ret < 0 ? ret : g_pubsubtest_res;
 }
 
 static int test_single(void)
@@ -239,7 +254,7 @@ static int test_single(void)
   bool updated;
   int afd;
   int sfd;
-  int ret;
+  int ret = ERROR;
 
   test_note("try single-topic support");
 
@@ -257,6 +272,7 @@ static int test_single(void)
   sfd = orb_subscribe(ORB_ID(orb_test));
   if (sfd < 0)
     {
+      orb_unadvertise(afd);
       return test_fail("subscribe failed: %d", errno);
     }
 
@@ -266,23 +282,27 @@ static int test_single(void)
 
   if (OK != orb_copy(ORB_ID(orb_test), sfd, &sub_sample))
     {
-      return test_fail("copy(1) failed: %d", errno);
+      test_fail("copy(1) failed: %d", errno);
+      goto out;
     }
 
   if (sample.val != sub_sample.val)
     {
-      return test_fail("copy(1) mismatch: %d expected %d",
-                       sub_sample.val, sample.val);
+      test_fail("copy(1) mismatch: %d expected %d",
+                sub_sample.val, sample.val);
+      goto out;
     }
 
   if (OK != orb_check(sfd, &updated))
     {
-      return test_fail("check(1) failed");
+      test_fail("check(1) failed");
+      goto out;
     }
 
   if (updated)
     {
-      return test_fail("spurious updated flag");
+      test_fail("spurious updated flag");
+      goto out;
     }
 
   /* check second publish */
@@ -291,45 +311,56 @@ static int test_single(void)
 
   if (OK != orb_publish(ORB_ID(orb_test), afd, &sample))
     {
-      return test_fail("publish failed");
+      test_fail("publish failed");
+      goto out;
     }
 
   if (OK != orb_check(sfd, &updated))
     {
-      return test_fail("check(2) failed");
+      test_fail("check(2) failed");
+      goto out;
     }
 
   if (!updated)
     {
-      return test_fail("missing updated flag");
+      test_fail("missing updated flag");
+      goto out;
     }
 
   if (OK != orb_copy(ORB_ID(orb_test), sfd, &sub_sample))
     {
-      return test_fail("copy(2) failed: %d", errno);
+      test_fail("copy(2) failed: %d", errno);
+      goto out;
     }
 
   if (sample.val != sub_sample.val)
     {
-      return test_fail("copy(2) mismatch: %d expected %d",
-                       sub_sample.val, sample.val);
+      test_fail("copy(2) mismatch: %d expected %d",
+                sub_sample.val, sample.val);
+      goto out;
     }
 
   /* unadvertise and out */
 
+out:
   ret = orb_unadvertise(afd);
   if (ret != OK)
     {
-      return test_fail("orb_unadvertise failed: %i", ret);
+      test_fail("orb_unadvertise failed: %i", ret);
     }
 
   ret = orb_unsubscribe(sfd);
   if (ret != OK)
     {
-      return test_fail("orb_unsubscribe failed: %i", ret);
+      test_fail("orb_unsubscribe failed: %i", ret);
     }
 
-  return test_note("PASS single-topic test");
+  if (ret == OK)
+    {
+      test_note("PASS single-topic test");
+    }
+
+  return ret;
 }
 
 static int test_multi_inst10(void)
@@ -337,6 +368,7 @@ static int test_multi_inst10(void)
   const int max_inst = 10;
   int sfd[max_inst];
   int afd[max_inst];
+  int ret = ERROR;
   int i;
   int j;
 
@@ -351,7 +383,8 @@ static int test_multi_inst10(void)
     {
       if (OK == orb_exists(ORB_ID(orb_test), i))
         {
-          return test_fail("sub %d is advertised", i);
+          test_fail("sub %d is advertised", i);
+          goto out_with_sfd;
         }
     }
 
@@ -361,9 +394,16 @@ static int test_multi_inst10(void)
     {
       afd[i] = orb_advertise_multi_queue_persist(ORB_ID(orb_test),
                                                  NULL, &i, 1);
+    }
+
+  /* verify advertised yet */
+
+  for (i = 0; i < max_inst; i++)
+    {
       if (OK != orb_exists(ORB_ID(orb_test), i))
         {
-          return test_fail("sub %d advertise failed", i);
+          test_fail("sub %d advertise failed", i);
+          goto out_with_sfd;
         }
     }
 
@@ -393,7 +433,8 @@ static int test_multi_inst10(void)
               orb_check(sfd[sub_instance], &updated);
               if (!updated)
                 {
-                  return test_fail("sub %d not updated", sub_instance);
+                  test_fail("sub %d not updated", sub_instance);
+                  goto out;
                 }
               else
                 {
@@ -401,14 +442,16 @@ static int test_multi_inst10(void)
 
                   if (orb_copy(ORB_ID(orb_test), sfd[sub_instance], &sample))
                     {
-                      return test_fail("sub %d copy failed", sub_instance);
+                      test_fail("sub %d copy failed", sub_instance);
+                      goto out;
                     }
                   else
                     {
                       if (sample.val != (j * sub_instance + i))
                         {
-                          return test_fail("sub %d invalid value %d",
-                                           sub_instance, sample.val);
+                          test_fail("sub %d invalid value %d",
+                                    sub_instance, sample.val);
+                          goto out;
                         }
                     }
                 }
@@ -416,14 +459,26 @@ static int test_multi_inst10(void)
         }
     }
 
-  /* force unsubscribe all, then repeat */
+  ret = OK;
 
+out:
+  for (i = 0; i < max_inst; i++)
+    {
+      orb_unadvertise(afd[i]);
+    }
+
+out_with_sfd:
   for (i = 0; i < max_inst; i++)
     {
       orb_unsubscribe(sfd[i]);
     }
 
-  return test_note("PASS orb 10-instances");
+  if (ret == OK)
+    {
+      test_note("PASS orb 10-instances");
+    }
+
+  return ret;
 }
 
 static int test_multi(int *afds, int *sfds)
@@ -605,7 +660,7 @@ static int test_unadvertise(int *afds)
   return OK;
 }
 
-static int pub_test_multi2_entry(int argc, char *argv[])
+static FAR void *pub_test_multi2_entry(FAR void *arg)
 {
   struct orb_test_medium_s data_topic;
   const int num_instances = 3;
@@ -615,6 +670,7 @@ static int pub_test_multi2_entry(int argc, char *argv[])
   int num_messages = 50 * num_instances;
   int i;
 
+  (void)arg;
   memset(&data_topic, '\0', sizeof(data_topic));
   for (i = 0; i < num_instances; ++i)
     {
@@ -650,7 +706,7 @@ static int pub_test_multi2_entry(int argc, char *argv[])
       orb_unadvertise(orb_pub[i]);
     }
 
-  return OK;
+  return NULL;
 }
 
 static int test_multi2(void)
@@ -659,7 +715,10 @@ static int test_multi2(void)
   int orb_data_fd[num_instances];
   int orb_data_next     = 0;
   orb_abstime last_time = 0;
-  int pubsub_task;
+  struct sched_param param;
+  pthread_attr_t attr;
+  pthread_t pubsub_task;
+  int ret = ERROR;
   int i;
 
   test_note("Testing multi-topic 2 test (queue simulation)");
@@ -675,16 +734,19 @@ static int test_multi2(void)
 
   /* launch the publisher thread */
 
-  pubsub_task = task_create("uorb_test_multi",
-                            SCHED_PRIORITY_MAX - 5,
-                            CONFIG_UORB_STACKSIZE,
-                            pub_test_multi2_entry,
-                            NULL);
-  if (pubsub_task < 0)
+  pthread_attr_init(&attr);
+  param.sched_priority = sched_get_priority_max(0) - 5;
+  pthread_attr_setschedparam(&attr, &param);
+
+  if (pthread_create(&pubsub_task, &attr, pub_test_multi2_entry, NULL) < 0)
     {
-      return test_fail("failed launching task");
+      pthread_attr_destroy(&attr);
+      test_fail("failed launching task");
+      goto out;
     }
 
+  pthread_attr_destroy(&attr);
+
   /* loop check update and copy new data */
 
   while (!g_thread_should_exit)
@@ -705,13 +767,15 @@ static int test_multi2(void)
           if (OK != orb_copy(ORB_ID(orb_test_medium_multi),
                              orb_data_cur_fd, &msg))
             {
-              return test_fail("copy failed: %d", errno);
+              test_fail("copy failed: %d", errno);
+              goto out;
             }
 
           if (last_time >= msg.timestamp && last_time != 0)
             {
-              return test_fail("Timestamp not increasing! (%" PRIu64
-                               " >= %" PRIu64 ")", last_time, msg.timestamp);
+              test_fail("Timestamp not increasing! (%" PRIu64
+                        " >= %" PRIu64 ")", last_time, msg.timestamp);
+              goto out;
             }
 
           last_time     = msg.timestamp;
@@ -719,12 +783,20 @@ static int test_multi2(void)
         }
     }
 
+  ret = OK;
+
+out:
   for (i = 0; i < num_instances; ++i)
     {
       orb_unsubscribe(orb_data_fd[i]);
     }
 
-  return test_note("PASS multi-topic 2 test (queue simulation)");
+  if (ret == OK)
+    {
+      test_note("PASS multi-topic 2 test (queue simulation)");
+    }
+
+  return ret;
 }
 
 int test_queue(void)
@@ -737,6 +809,7 @@ int test_queue(void)
   int instance = 0;
   int ptopic;
   int sfd;
+  int ret = ERROR;
   int i;
 
   test_note("Testing orb queuing");
@@ -766,52 +839,60 @@ int test_queue(void)
     ORB_ID(orb_test_medium_queue), &sample, &instance, queue_size);
   if (ptopic < 0)
     {
+      orb_unsubscribe(sfd);
       return test_fail("advertise failed: %d", errno);
     }
 
   orb_check(sfd, &updated);
   if (!updated)
     {
-      return test_fail("update flag not set");
+      test_fail("update flag not set");
+      goto out;
     }
 
   if (OK != orb_copy(ORB_ID(orb_test_medium_queue), sfd, &sub_sample))
     {
-      return test_fail("copy(1) failed: %d", errno);
+      test_fail("copy(1) failed: %d", errno);
+      goto out;
     }
 
   if (sub_sample.val != sample.val)
     {
-      return test_fail("copy(1) mismatch: %d expected %d",
-                       sub_sample.val, sample.val);
+      test_fail("copy(1) mismatch: %d expected %d",
+                sub_sample.val, sample.val);
+      goto out;
     }
 
   orb_check(sfd, &updated);
   if (updated)
     {
-      return test_fail("spurious updated flag");
+      test_fail("spurious updated flag");
+      goto out;
     }
 
 #define CHECK_UPDATED(element) \
   orb_check(sfd, &updated); \
   if (!updated) \
     { \
-      return test_fail("update flag not set, element %i", element); \
+      test_fail("update flag not set, element %i", element); \
+      goto out; \
     }
 
 #define CHECK_NOT_UPDATED(element) \
   orb_check(sfd, &updated); \
   if (updated) \
     { \
-      return test_fail("update flag set, element %i", element); \
+      test_fail("update flag set, element %i", element); \
+      goto out; \
     }
 
 #define CHECK_COPY(i_got, i_correct) \
   orb_copy(ORB_ID(orb_test_medium_queue), sfd, &sub_sample); \
   if (i_got != i_correct) \
     { \
-      return test_fail("got wrong element from the queue (got %i," \
-                       "should be %i)", i_got, i_correct); \
+      test_fail("got wrong element from the queue (got %i," \
+                "should be %i)", i_got, i_correct); \
+      goto out; \
     }
 
   /* no messages in the queue anymore */
@@ -867,13 +948,21 @@ int test_queue(void)
 #undef CHECK_UPDATED
 #undef CHECK_NOT_UPDATED
 
+  ret = OK;
+
+out:
   orb_unadvertise(ptopic);
   orb_unsubscribe(sfd);
 
-  return test_note("PASS orb queuing");
+  if (ret == OK)
+    {
+      test_note("PASS orb queuing");
+    }
+
+  return ret;
 }
 
-static int pub_test_queue_entry(int argc, char *argv[])
+static FAR void *pub_test_queue_entry(FAR void *arg)
 {
   const int queue_size = 50;
   struct orb_test_medium_s t;
@@ -882,13 +971,15 @@ static int pub_test_queue_entry(int argc, char *argv[])
   int instance = 0;
   int ptopic;
 
+  (void)arg;
   memset(&t, '\0', sizeof(t));
   ptopic = orb_advertise_multi_queue_persist(
     ORB_ID(orb_test_medium_queue_poll), &t, &instance, queue_size);
   if (ptopic < 0)
     {
       g_thread_should_exit = true;
-      return test_fail("advertise failed: %d", errno);
+      test_fail("advertise failed: %d", errno);
+      return NULL;
     }
 
   ++t.val;
@@ -915,17 +1006,18 @@ static int pub_test_queue_entry(int argc, char *argv[])
   usleep(100 * 1000);
   g_thread_should_exit = true;
   orb_unadvertise(ptopic);
-
-  return 0;
+  return NULL;
 }
 
 static int test_queue_poll_notify(void)
 {
   struct pollfd fds[1];
   struct orb_test_medium_s t;
+  struct sched_param param;
+  pthread_attr_t attr;
   bool updated;
   int next_expected_val = 0;
-  int pubsub_task;
+  pthread_t pubsub_task;
   int sfd;
 
   test_note("Testing orb queuing (poll & notify)");
@@ -951,17 +1043,19 @@ static int test_queue_poll_notify(void)
 
   g_thread_should_exit = false;
 
-  pubsub_task = task_create("uorb_test_queue",
-                            SCHED_PRIORITY_MIN + 5,
-                            CONFIG_UORB_STACKSIZE,
-                            pub_test_queue_entry,
-                            NULL);
+  pthread_attr_init(&attr);
+  param.sched_priority = sched_get_priority_min(0) + 5;
+  pthread_attr_setschedparam(&attr, &param);
 
-  if (pubsub_task < 0)
+  if (pthread_create(&pubsub_task, &attr, pub_test_queue_entry, NULL) < 0)
     {
-      return test_fail("failed launching task");
+      orb_unsubscribe(sfd);
+      pthread_attr_destroy(&attr);
+      return test_fail("failed test queue task");
     }
 
+  pthread_attr_destroy(&attr);
+
   fds[0].fd     = sfd;
   fds[0].events = POLLIN;
 
@@ -977,10 +1071,12 @@ static int test_queue_poll_notify(void)
               break;
             }
 
+          orb_unsubscribe(sfd);
           return test_fail("poll timeout");
         }
       else if (poll_ret < 0 && errno != EINTR)
         {
+          orb_unsubscribe(sfd);
           return test_fail("poll error (%d, %d)", poll_ret, errno);
         }
 
@@ -989,6 +1085,7 @@ static int test_queue_poll_notify(void)
           orb_copy(ORB_ID(orb_test_medium_queue_poll), sfd, &t);
           if (next_expected_val != t.val)
             {
+              orb_unsubscribe(sfd);
               return test_fail("copy mismatch: %d expected %d",
                                t.val, next_expected_val);
             }
@@ -999,6 +1096,7 @@ static int test_queue_poll_notify(void)
 
   if (g_num_messages_sent != next_expected_val)
     {
+      orb_unsubscribe(sfd);
       return test_fail("number of sent and received messages mismatch"
                        " (sent: %i, received: %i)",
                        g_num_messages_sent, next_expected_val);
