@@ -23,8 +23,8 @@
  ****************************************************************************/
 
 #include <stdarg.h>
-#include <stdio.h>
 #include <syslog.h>
+#include <nuttx/streams.h>
 #include "monkey_log.h"
 
 /****************************************************************************
