@@ -346,7 +346,7 @@ static int user_main(int argc, char *argv[])
 #endif
 
 #if defined(CONFIG_ARCH_FPU) && !defined(CONFIG_TESTING_OSTEST_FPUTESTDISABLE) && \
-    !defined(CONFIG_BUILD_KERNEL)
+    defined(CONFIG_BUILD_FLAT)
       /* Check that the FPU is properly supported during context switching */
 
       printf("\nuser_main: FPU test\n");
@@ -470,7 +470,7 @@ static int user_main(int argc, char *argv[])
       pthread_rwlock_cancel_test();
       check_test_memory_usage();
 
-#if CONFIG_PTHREAD_CLEANUP_STACKSIZE > 0
+#if CONFIG_TLS_NCLEANUP > 0
       /* Verify pthread cancellation cleanup handlers */
 
       printf("\nuser_main: pthread_cleanup test\n");
@@ -592,23 +592,23 @@ static int user_main(int argc, char *argv[])
 
 #if defined(CONFIG_ARCH_HAVE_FORK) && defined(CONFIG_SCHED_WAITPID) && \
    !defined(CONFIG_ARCH_SIM)
-#ifndef CONFIG_BUILD_KERNEL
       printf("\nuser_main: vfork() test\n");
       vfork_test();
-#else
-      /* REVISIT: The issue with vfork() is on the kernel side, fix the issue
-       * and re-enable this test with CONFIG_BUILD_KERNEL
-       */
-
-      printf("\nuser_main: vfork() test DISABLED (CONFIG_BUILD_KERNEL)\n");
-#endif
 #endif
 
-#ifdef CONFIG_SMP_CALL
+#if defined(CONFIG_SMP_CALL) && defined(CONFIG_BUILD_FLAT)
       printf("\nuser_main: smp call test\n");
       smp_call_test();
 #endif
 
+#if defined(CONFIG_SCHED_EVENTS) && defined(CONFIG_BUILD_FLAT)
+      /* Verify nxevent */
+
+      printf("\nuser_main: nxevent test\n");
+      nxevent_test();
+      check_test_memory_usage();
+#endif
+
       /* Compare memory usage at time ostest_main started until
        * user_main exits.  These should not be identical, but should
        * be similar enough that we can detect any serious OS memory
