@@ -275,6 +275,12 @@ FAR const char *wapi_ioctl_command_name(int cmd)
     case SIOCSIWPTAPRIO:
       return "SIOCSIWPTAPRIO";
 
+    case SIOCSIWPMKSA:
+      return "SIOCSIWPMKSA";
+
+    case SIOCGIWPMKSA:
+      return "SIOCGIWPMKSA";
+
     default:
       snprintf(g_ioctl_command_namebuf, WAPI_IOCTL_COMMAND_NAMEBUFSIZ,
                "0x%x", cmd);
@@ -505,7 +511,7 @@ int wapi_save_config(FAR const char *ifname,
       goto errout;
     }
 
-  fd = open(confname, O_RDWR | O_CREAT | O_TRUNC);
+  fd = open(confname, O_RDWR | O_CREAT | O_TRUNC, 0644);
   if (fd < 0)
     {
       ret = -errno;
